#
#  DEBUGGER.X
#
#  Testhilfe zur Erprobung hardwarenaher Funktion in der Zielharware
#  Nheres im Handbuch unter "Programmbeispiele"
#
#  Das Programm ist fr einen 80535 Controller konfiguriert. Soll es auf einem
#  8051 laufen, mu die Datei serial2.x (interruptgesteuerte Zeichen Ein-/
#  Ausgabe) und die Initilisierung der seriellen Schnittstelle in der Funktion
#  #MainProgram gendert werden.
#
#  Copyright (C) 1994 Markus Bauer
#



INCLUDE stddef.x                 # allgemeine Konstanten
INCLUDE chrconst.x               # Zeichenkonstanten (Steuerzeichen)

INCLUDE vardef.x                 # automatische Variablenplazierung
INCLUDE r8051.x                  # 8051 SFR Definitionen
INCLUDE int.x                    # Aliase fr Interrupts
INCLUDE serial2.x                # Serial I/O, gepuffert

INCLUDE xconvert.x               # ASCII <-> Binr Konvertierung
INCLUDE print.x                  # Ausgabe von Strings vom Stapel
INCLUDE dfunc.x                  # erweiterte 16 Bit Arithmetik

DFH SBIT BD                      # 1 = Baudrate 4800/9600 bei 80535

#
#  Die zu testenden Funktionen sollten hier includiert werden.
#






#
#  Konstanten des Debug-Programms
#

07FFHL LCONSTANT #HiAddress      # Ende des Debug Speichers
30H    CONSTANT  #StackBase      # erstes Byte des Parameterstacks, der Wert
                                 # dieser Konstanten mu gendert werden,
                                 # wenn das Programm nicht die Standard-
                                 # speicheraufteilung verwendet.

#
#  Variablen des Debug Programms
#

XBYTE            #CharCount      # Anz. Zeichen in #LineBuf
80L XDATA        #LineBuf        # Eingegebene Zeile
XWORD            #SymTabPtr      # Schreibzeiger Symboltabelle
0L XDATA         #SymbolTable    # Symboltabelle bis Speicherende
0 FIXVAR         #PStackPtr      # Stackpointer des Rechenstapels, MCS-51
                                 # spezifisch

#
#  Funktionen des Debug Programms
#



: #LineBuf! ( char index -> )
#
#  Speichert ein Zeichen in den Pufferbereich #LineBuf
#

   0 SWAP #LineBuf D+ X!

;


: #LineBuf@ ( index -> char )
#
#  Liest ein Zeichen aus dem Pufferbereich #LineBuf
#

   0 SWAP #LineBuf D+ X@

;


: #CharCount@ ( -> n )
#
#  Legt die Lnge der Eingangszeile auf den Stapel
#

   #CharCount X@

;



: #CharCount! ( n -> )
#
#  Speichert einen Wert in die Variable #CharCount
#

   #CharCount X!

;



: #GetToken ( -> )
#
#  Liest Zeichen durch GETCHAR, bis ein CR oder Blanc eingegeben wurde.
#  Die Zeichen werden in #LineBuf abgegegt. Nicht druckbare Zeichen werden
#  ignoriert. #GetToken schreibt die Anzahl der gelesenen Zeichen in die
#  Variable #CharCount.
#

   0 #CharCount!                         # Lsche Zeichenzhler
   BEGIN
      BEGIN                              # Lese Zeichen,
         GETCHAR
      DUP CR = OVER BLANC = OR NOT WHILE # ... bis CR oder BLANC eingegeben
         DUP BLANC > IF                  # druckbares Zeichen ?
            #CharCount@ 80 < IF          # noch Platz im Puffer ?
               DUP PUTCHAR               # Echo an das Terminal
               #CharCount@ #LineBuf!     # lege es im Pufferspeicher ab
               #CharCount@ 1 +           # Zhler erhhen
               #CharCount!
            ELSE
               DROP                      # Zeichen kann nicht gespeichert
               BELL PUTCHAR              # werden, Piepsen
            ENDIF
         ELSE
            DROP                         # nicht druckbare Zeichen vergessen
         ENDIF
      REPEAT
      DUP CR = IF                        # bei CR auch
         LF PUTCHAR                      # LF ausgeben
      ENDIF
      PUTCHAR                            # Zeichen (CR oder BLANC) ausgeben
   #CharCount@ UNTIL                     # Nochmal, wenn Zeichenlnge == 0

;





: #IncSymTabPtr ( n -> )
#
# Erhht den Schreibzeiger der Symboltabelle um den auf dem Stapel
# bergebenen 8-Bit Wert
#

   0 SWAP                    # erweitere auf 16 Bit
   #SymTabPtr DX@ D+         # berechne neuen Wert
   DDUP #HiAddress D< IF     # noch innerhalb des zulssigen Bereichs ?
      #SymTabPtr DX!         # dann speichere,
   ELSE
      DDROP                  # sonst lasse den alten Wert
   ENDIF

;



: #StoreAddress ( -> )
#
# Interpretiert den gelesenen String als Adresse der Symboltabelle und
# trgt diese in die Tabelle ein
#

   4 DO                      # lege die ersten vier Zeichen der Zeile
      I #LineBuf@            # in umgekehrter Reihenfolge auf den
   -LOOP                     # Stapel

   CharsToWord               # wandle die Adresse in eine 16 Bit Zahl
   #SymTabPtr DX@ DX!        # speichere die Adresse in der Tabelle
   2 #IncSymTabPtr           # schalte den Zeiger weiter

;



: #RemoveAddress ( -> )
#
# Entfernt die zuletzt gespeicherte Adresse aus der Symboltabelle
#

   #SymTabPtr DX@   2L D-   #SymTabPtr DX!

;



: #StoreName ( -> )
#
# Interpretiert den gelesenen String als Name der Symboltabelle und
# trgt diese in die Tabelle ein
#

   #CharCount@               # ermittle die Anzahl Zeichen des Namens
   #SymTabPtr DX@ X!         # speichere das Lngenbyte in der Tabelle
   1 #IncSymTabPtr           # schalte den Zeiger weiter

   0                         # Erster Indexwert
   #CharCount@ DO            # Anzahl Zeichen Mal
      DUP #LineBuf@          # kopiere ein Zeichen in die Tabelle
      #SymTabPtr DX@ X!
      1 #IncSymTabPtr        # schalte den Zeiger weiter
      1 +
   -LOOP
   DROP                      # Indexwert

;



: #LoadSymbolTable ( -> )
#
#  Erstellt die Symboltabelle. Es werden Zeilen eingelesen, bis ein Punkt in
#  der ersten Spalte das Ende der Symboltabelle markiert
#

   #SymbolTable #SymTabPtr DX!         # initialisiere den Schreibzeiger
   BEGIN                               # Lese Symboltabelle ...
      #GetToken                        # 1. Wort, ist "." oder Adresse
   0 #LineBuf@ '.' != WHILE            # Breche ab, wenn 1. Zeichen = "."
      #StoreAddress                    # Trage die Adresse ein
      #GetToken                        # 2. Wort, Symbolname
      0 #LineBuf@ '#' != IF            # Wenn das Symbol nicht mit # beginnt
         #StoreName                    # ... trage es ein
      ELSE
         #RemoveAddress                # ... oder entferne die Adresse wieder
      ENDIF
   REPEAT                              # nchste Zeile

;




: #CompareName ( adr16 -> flag )
#
#  Vergleicht den Inhalt der Eingangszeile mit dem String, der bei der
#  externen Speicheradresse adr16 mit einem Lngenbyte beginnt. flag wird
#  TRUE, wenn beide Strings gleich sind
#

   DDUP X@ #CharCount@ = IF          # sind beide Strings gleich lang ?
      1L D+                          # Adresse des 1.Zeichens in der Tabelle
      TRUE ROT ROT                   # lege unter die Adresse
      #CharCount@ DO                 # dann vergleiche alle Zeichen
         DDUP 0 I D+ X@              # Zeichen der Tabelle
         I #LineBuf@                 # Zeichen in derEingangszeile
         = NOT IF                    # wenn die nicht gleich sind ...
            ROT DROP                 # ... erzetze Ergebnis durch FALSE
            FALSE ROT ROT
         ENDIF
      -LOOP
      DDROP                          # entferne die Adresse
   ELSE
      DDROP FALSE                    # bei ungleicher Lnge knnen die
   ENDIF                             # Strings nicht gleich sein

;



: #SearchInSymbolTable ( --> adr16 )
#
#  Durchsucht die Symboltabelle nach dem Bezeichner, der in der Eingabezeile
#  steht. Legt die Adresse des Lngenbytes des gefundenen Strings auf den
#  Stapel. Wurde der String nicht gefunden, ist adr16 > #SymTabPtr.
#

   #SymbolTable 2L D+                # Adresse des ersten Strings
   BEGIN
      DDUP #SymTabPtr DX@ D< IF      # sind wir noch im zulssigen Bereich ?
         DDUP #CompareName NOT       # ... dann vergleichen
      ELSE
         FALSE                       # ... auerhalb der Tabelle
      ENDIF
   WHILE                             # fertig, wenn gefunden oder auerhalb
      DDUP X@ 0 SWAP D+              # Stringlnge -> 16 Bit und addieren
      3L D+                          # Lnge des Adressfeldes und des
   REPEAT                            # Lngenfeldes berspringen

;


: #Call ( adr16 -> )
#
# Springt zu der durch die 16 Bit-Adresse bezeichneten compilierten Funktion
# Funktioniert in dieser Form nur fr MCS-51 Microcontroller
#

  >R DROP          # Lo Byte der Adresse auf den Returnstack
  >R DROP          # Hi Byte der Adresse auf den Returnstack
                   # Der RET Befehl am Ende der Funktion fhrt den
                   # Sprung aus.
;



: D ( -> )
#
# Listet den gesamten internen Speicher als Hexdump
#

   0                                # beginne bei Speicherzelle 0
   16 DO                            # 16 Zeilen ...
      DUP ByteToChars 2 PRINT       # Adresse ausgeben
      ':' PUTCHAR                   # : nach Adresse
      16 DO                         # 16 Bytes ...
         BLANC PUTCHAR
         DUP @ ByteToChars 2 PRINT  # Inhalt der Speicherzelle
         1 +                        # nchste Speicherzelle
      -LOOP
      LF CR 2 PRINT                 # Zeile abschlieen
   -LOOP
   DROP                             # Letzte Adresse vom Stapel nehmem

;


: #MainProgram ( -> )
#
#  Hauptprogramm des Mini-Interpreters. Liest zunchst die Symboltabelle ein,
#  liest und bearbeitet dann in einer Endlosschleife alle Befehle
#

   52H  SCON!                        # Serielle Schnittstelle initialisieren
   TRUE BD!                          # Interne Baudratenerzeugung 80535
   PCON@ 80H OR PCON!                # doppelte Baudrate (9600)

   sInitBuffers                      # Schnittstellentreiber initialisieren

   #LoadSymbolTable
   10 13 '.' 'k' 'o' 5 PRINT

   BEGIN
      #GetToken                      # Einen Befehl einlesen und bearbeiten
      #CharCount@ IF
         #SearchInSymbolTable
         DDUP #SymTabPtr DX@ D< IF   # in der Symboltabelle gefunden ?
            2L D- DX@                # gespeicherte Adresse aus der Tabelle
            #Call                    # holen und ausfhren
         ELSE
            DDROP
            #CharCount@ 2 = IF       # 2 Zeichen lang ?
               1 #LineBuf@           # Als Hex Byte interpretieren und
               0 #LineBuf@           # den Wert auf den Stapel legen
               CharsToByte
            ELSE
               13 10 '?' 3 PRINT     # Fehlermeldung
            ENDIF
         ENDIF
      ENDIF
   POWER_OFF UNTIL                   # im Interpreter

;


: imem ( -> )
#
# Listet den gesamten internen Speicher als Hexdump
#

   0                                # beginne bei Speicherzelle 0
   16 DO                            # 16 Zeilen ...
      DUP ByteToChars 2 PRINT       # Adresse ausgeben
      ':' PUTCHAR                   # : nach Adresse
      16 DO                         # 16 Bytes ...
         BLANC PUTCHAR
         DUP @ ByteToChars 2 PRINT  # Inhalt der Speicherzelle
         1 +                        # nchste Speicherzelle
      -LOOP
      LF CR 2 PRINT                 # Zeile abschlieen
   -LOOP
   DROP                             # Letzte Adresse vom Stapel nehmem

;


: stack ( -> )
#
# Listet den Rechenstapel in Hex. Funktioniert in dieser Form nur bei
# MCS-51 Controllern.
#

   0 @ #StackBase 1 - -             # Ermittle den Stand des Parameter-
                                    # Stackpointers.
   DUP IF                           # Liegt etwas auf dem Stapel ?
      #StackBase SWAP               # Ja, lege Adresse des 1.Bytes darunter
      DO                            # Anzahl Bytes mal
         DUP @ ByteToChars 2 PRINT  # Inhalt des Stapelplatzes
         1 +                        # nchster Stapelplatz
         BLANC PUTCHAR              # Leerzeichen anfgen
      -LOOP
   ENDIF
   LF CR '.' 3 PRINT                # Zeile abschlieen
   DROP                             # Letzte Adresse vom Stapel nehmem

;


: cls ( -> )
#
#  Lscht den Rechenstapel. Funktioniert in dieser Form nur bei MCS-51
#  Controllern.

   #StackBase 1 - #PStackPtr!

;




: _Compile_Also
#
#  Dummy-Funktion mit Aufrufen aller Funktionen, die im Programm nirgends
#  aufgerufen werden, aber mit in das EPROM compiliert werden sollen.
#

   imem             # Darstellen des gesamten internen Speichers
   stack            # Darstellen des Rechenstapels
   cls              # Lschen des Rechenstapels

                    # ... Namen der zu testenden Funktionen hier einfgen

   #MainProgram     # Hauptprogramm
;




#
#  Erzeugen des EPROM-Files
#

$IFERR BYE $ENDIF                 # Breche ab, wenn Fehler auftraten

LOG           TEST.LOG            # die  Datei enthlt die Symboltabelle fr den
0030HL        COMPILE             # Debugger
NOLOG

RESET         #MainProgram        # Erzeugen der HEX - Datei
4 INTERRUPT   _sInterruptService
WRITE         TEST.HEX

BYE                               # und tsch ...


