#
# DFUNC.X definiert einige arithmetische und Vergleichsfunktionen fr
# 16-Bit Zahlen
#
#    DDUP     ( d1 -> d1 d1 )      Stackmanipulationen mit 16-Bit Zahlen
#    DSWAP    ( d1 d2 -> d2 d1 )
#    DDROP    ( d1 -> )
#    DSTACK@  ( dn dn-1 ... d0 n -> dn dn-1 ... d0 dn )
#    DSTACK!  ( dn dn-1 ... d0 d n -> d dn-1 ... d0 )
#    DOVER    ( d1 d2 -> d1 d2 d1 )
#
#    D>=      ( d1 d2 -> flag )    FFH, wenn d1 >= d2, sonst 0
#    D<=      ( d1 d2 -> flag )    FFH, wenn d1 <= d2, sonst 0
#    D<       ( d1 d2 -> flag )    FFH, wenn d1 < d2, sonst 0
#    D>       ( d1 d2 -> flag )    FFH, wenn d1 > d2, sonst 0
#    D=       ( d1 d2 -> flag )    FFH, wenn d1 = d2, sonst 0
#
#    DMINUS   ( d1 -> d1 )         Vorzeichenwechsel
#    D/       ( d1 d2 -> d3 )      16 Bit Division ohne Vorzeichen
#
#    Copyright (C) 1994 Markus Bauer
#



: DDUP     ( d1 -> d1 d1 )

#   dupliziert eine 16-Bit Zahl auf dem Rechenstapel

   OVER OVER

;



: DSTACK@  ( dn dn-1 ... d0 n -> dn dn-1 ... d0 dn )

#   STACK@ fr 16 Bit Zahlen. Der 8-Bit Index n whlt die 16-Bit Zahl
#   auf dem Stapel aus, die auf den Stapel gelegt werden soll

   1 + 2 * DUP                       # Stack Offset zum gewnschten Hi Byte
   STACK@ SWAP 1 - STACK@            # 16 Bit Zahl auf den Stapel legen

;


: DSTACK!  ( dn dn-1 ... d0 d n -> d dn-1 ... d0 )

#   STACK! fr 16 Bit Zahlen. Der 8-Bit Index n whlt die 16-Bit Zahl
#   auf dem Stapel aus, die im Stapel gespeichert werden soll

   1 + 2 *                           # Stack Offset zum gewnschten Hi Byte
   SWAP OVER STACK!                  # 16 Bit Zahl im Stapel speichern
   1 - STACK!

;


: DOVER    ( d1 d2 -> d1 d2 d1 )

#   OVER fr 16 Bit Zahlen

   1 DSTACK@

;


: DSWAP    ( d1 d2 -> d2 d1 )

#   vertauscht zwei 16-Bit Zahlen auf dem Rechenstapel

   >R DROP ROT ROT
   R>      ROT ROT

;



: DDROP    ( d1 -> )

#   nimmt eine 16 Bit Zahl vom Rechenstapel

   DROP DROP

;


: D>=      ( d1 d2 -> flag )

#   nimmt zwei 16 Bit Zahlen vom Rechenstapel und vergleicht diese.
#   Ist d1 >= d2, wird FFH auf den Stapel gelegt, sonst 0

   D- DROP 80H <

;


: D<=      ( d1 d2 -> flag )

#   nimmt zwei 16 Bit Zahlen vom Rechenstapel und vergleicht diese.
#   Ist d1 <= d2, wird FFH auf den Stapel gelegt, sonst 0

   DSWAP D>=

;


: D<       ( d1 d2 -> flag )

#   nimmt zwei 16 Bit Zahlen vom Rechenstapel und vergleicht diese.
#   Ist d1 < d2, wird FFH auf den Stapel gelegt, sonst 0

   D- DROP 80H >=

;


: D>      ( d1 d2 -> flag )

#   nimmt zwei 16 Bit Zahlen vom Rechenstapel und vergleicht diese.
#   Ist d1 > d2, wird FFH auf den Stapel gelegt, sonst 0

   DSWAP D<

;


: D=       ( d1 d2 -> flag )

#   nimmt zwei 16 Bit Zahlen vom Rechenstapel und vergleicht diese.
#   Ist d1 = d2, wird FFH auf den Stapel gelegt, sonst 0

   D- OR 0 =

;



: DMINUS    ( d1 -> d1 )

#   wechselt das Vorzeichen einer 16 Bit Zahl

   0 ROT ROT
   0 ROT ROT
   D-

;

: D/_$$$001

#   Hilfsfunktion fr D/. Fhrt die eigentliche Division aus

   BEGIN           # Multipliziere d2 mit 2 ...
      DSHL         # h = h * 2                      d1 d2 d3 h
      2 DSTACK@    # d2 = d2 * 2                    d1 d2 d3 h
      DSHL
      2 DSTACK!
      3 DSTACK@    # Hole d1 auf den Stapel         d1 d2 d3 h  d1
      3 DSTACK@    # Hole d2 auf den Stapel         d1 d2 d3 h  d1 d2
      OVER 80H AND # Bestimme, ob das hchste Bit   d1 d2 d3 h  d1 d2 nn
      >R DROP      # von d2 gesetzt ist
   D< R> OR UNTIL  # ... bis (d1<d2) OR (d2>=8000H) d1 d2 d3 h
   BEGIN
      DDUP OR
   WHILE           # solange h != 0 ...
      3 DSTACK@    # Hole d1 auf den Stapel         d1 d2 d3 h  d1
      3 DSTACK@    # Hole d2 auf den Stapel         d1 d2 d3 h  d1 d2
      D>= IF       # Wenn d1 >= d2                  d1 d2 d3 h
         3 DSTACK@ # Bilde d1 = d1 - d2             d1 d2 d3 h  d1
         3 DSTACK@ #                                d1 d2 d3 h  d1 d2
         D-        #                                d1 d2 d3 h  diff
         3 DSTACK! #                                d1 d2 d3 h
         OVER      # d3 = d3 OR h
         4 STACK@
         OR
         3 STACK!
         DUP
         3 STACK@
         OR
         2 STACK!
      ENDIF
      DSHR         # h = h / 2                      d1 d2 d3 h
      2 DSTACK@    # d2 = d2 / 2                    d1 d2 d3 h
      DSHR
      2 DSTACK!
   REPEAT

;

: D/ ( d1 d2 -> d3 )

#   16 Bit Unsigned Division d3 = d1 / d2. Bentigt 14 Bytes Parameterstapel!

   DDUP OR IF            # d2 ungleich 0 ?                d1 d2
      0L                 # Ergebnis auf den Stapel        d1 d2 d3
      2 DSTACK@          # Hole d1 auf den Stapel         d1 d2 d3 d1
      2 DSTACK@          # Hole d2 auf den Stapel         d1 d2 d3 d1 d2
      D>= IF             # nur wenn d1 >= d2 dividieren   d1 d2 d3
         1L              # Hilfsvariable auf den Stapel   d1 d2 d3 h
         D/_$$$001       # Fhrt die Division aus         d1 d2 d3 h
         DDROP           # H wird nicht mehr bentigt     d1 d2 d3
      ENDIF
      1 DSTACK!          # Speichere Ergebnis             d3 d2 d3
      2 NDROP            # Rume den Stack auf            d3
   ELSE
      DDROP              # Bei Division / 0 bleibt d1 auf dem Stapel
   ENDIF

;

HIDE D/_$$$001           # D/_Divide ist nur fr lokalen Gebrauch
