#
#  SIGNED.X   8-Bit Arithmetik Funktionen mit Vorzeichenbercksichtigung
#
#  Copyright (C) 1994 Dipl.Ing.(FH) Markus Bauer
#
#  Enthalten sind folgende Funktionen:
#
#     NEGATIV ( n -> flag )      Nimmt die Zahl n vom Stapel. Ist diese
#                                negativ, wird 0FFH auf den Stapel gelegt,
#                                sonst 0.
#
#     ABS ( n -> n )             Ersetzt die Zahl n auf dem Stapel durch
#                                ihren Absolutbetrag.
#
#     S/  ( n1 n2 -> n3 )        nimmt die Zahlen n1 und n2 vom Stapel und
#                                legt den Quotienten n1 / n2 dort ab.
#                                Bercksichtigt das Vorzeichen.
#
#     S> ( n1 n2 -> flag )       nimmt die Zahlen n1 und n2 vom Stapel und
#                                vergleicht sie unter Bercksichtigung des
#                                Vorzeichens. Ist n1 > n2, wird FFH auf den
#                                Stapel gelegt, sonst 0.
#
#     S< ( n1 n2 -> flag )       nimmt die Zahlen n1 und n2 vom Stapel und
#                                vergleicht sie unter Bercksichtigung des
#                                Vorzeichens. Ist n1 < n2, wird FFH auf den
#                                Stapel gelegt, sonst 0
#
#     S<= ( n1 n2 -> flag )      nimmt die Zahlen n1 und n2 vom Stapel und
#                                vergleicht sie unter Bercksichtigung des
#                                Vorzeichens. Ist n1 <= n2, wird FFH auf den
#                                Stapel gelegt, sonst 0
#
#     S>= ( n1 n2 -> flag )      nimmt die Zahlen n1 und n2 vom Stapel und
#                                vergleicht sie unter Bercksichtigung des
#                                Vorzeichens. Ist n1 >= n2, wird FFH auf den
#                                Stapel gelegt, sonst 0
#
#
#



: NEGATIV ( n -> flag )

# nimmt die Zahl n vom Stapel. Ist diese negativ, wird 0FFH auf den Stapel
# gelegt, sonst 0.

   7FH >

;



: ABS ( n -> n )

# ersetzt die Zahl n auf dem Stapel durch ihren Absolutbetrag

   DUP NEGATIV IF
      MINUS
   ENDIF

;


: S/  ( n1 n2 -> n3 )

# nimmt die Zahlen n1 und n2 vom Stapel und legt den Quotienten n1 / n2
# dort ab. Bercksichtigt Vorzeichen.

   DUP NEGATIV IF            # ist n2 negativ ?
      MINUS                  # ja, wechsle das Vorzeichen
      SWAP                   # betrachte n1
      DUP NEGATIV IF         # ist n1 auch negativ ?
         MINUS               # ja, wechsle auch hier das Vorzeichen
         SWAP /              # dividiere (alte Reihenfolge)
      ELSE                   # n1 ist positiv
         SWAP /              # dividiere (alte Reihenfolge)
         MINUS               # Ergebnis wird negativ
      ENDIF
   ELSE                      # n2 ist positiv
      SWAP                   # betrachte n1
      DUP NEGATIV IF         # ist n1 negativ ?
         MINUS               # ja, wechsle das Vorzeichen
         SWAP /              # dividiere (alte Reihenfolge)
         MINUS               # Ergebnis wird negativ
      ELSE
         SWAP /              # dividiere (alte Reihenfolge)
      ENDIF
   ENDIF

;


: S>    ( n1 n2 -> flag )

# nimmt die Zahlen n1 und n2 vom Stapel und vergleicht sie unter
# Bercksichtigung des Vorzeichens. Ist n1 > n2, wird FFH auf den Stapel
# gelegt, sonst 0

   SWAP - NEGATIV

;


: S<     ( n1 n2 -> flag )

# nimmt die Zahlen n1 und n2 vom Stapel und vergleicht sie unter
# Bercksichtigung des Vorzeichens. Ist n1 < n2, wird FFH auf den Stapel
# gelegt, sonst 0

   - NEGATIV

;


: S<=    ( n1 n2 -> flag )

# nimmt die Zahlen n1 und n2 vom Stapel und vergleicht sie unter
# Bercksichtigung des Vorzeichens. Ist n1 <= n2, wird FFH auf den Stapel
# gelegt, sonst 0

   SWAP - NEGATIV NOT

;


: S>=    ( n1 n2 -> flag )

# nimmt die Zahlen n1 und n2 vom Stapel und vergleicht sie unter
# Bercksichtigung des Vorzeichens. Ist n1 >= n2, wird FFH auf den Stapel
# gelegt, sonst 0

   - NEGATIV NOT

;


