#
# STRINGS.X definiert Funktionen zur Ausgabe von Zeichenketten, die gemeinsam
# in einer CODETABLE definiert sind. STRINGS.X erwartet, da zuvor eine
# CODETABLE mit dem Namen "Text" definiert wurde, die alle Strings enthlt.
# Jeder String in Text mu durch ein Byte mit dem Wert 0 abgeschlossen sein.
# Zur Ausgabe der Zeichen verwenden die Funktionen einen Aufruf der Funktion
# PUTCHAR ( ch -> ). Diese mu also auch vor dem bersetzen von STRINGS.X
# definiert worden sein.
#
#   SkipText ( n -> n )    berspringt einen durch 0 abgeschlossenen String
#                          in der Tabelle Text
#   PrintText ( n -> )     gibt einen durch 0 abgeschlossenen String aus der
#                          Tabelle Text aus, beginnend mit dem Zeichen
#                          Text[n]
#   PrintString ( n -> )   gibt den String mit der Nummer n aus der Tabelle
#                          Text aus. n=0 bezeichnet den ersten String der
#                          Tabelle
#
#   Copyright (C) 1994   Markus Bauer
#


: SkipText ( n -> n )
#
#  berspringt einen durch 0 abgeschlossenen String in der CODETABLE Text
#  Auf dem Stapel wird ein Index auf ein Zeichen in Text erwartet. Dieser
#  wird erhht, bis ein 0-Byte in der Tabelle gefunden wird. SkipText
#  hinterlt auf dem Stapel einen Indexwert, der das erste Zeichen des
#  folgenden Strings in Text bezeichnet
#

                     # n liegt auf dem Stapel
   BEGIN
      DUP Text       # ermittle Text[n], erhalte n
   WHILE             # mache weiter fr Text[n] <> 0
      1 +            # nchstes Zeichen
   REPEAT
   1 +               # zum 1. Zeichen des Folgestrings

;

: PrintText ( n -> )
#
#  Gibt einen durch 0 abgeschlossenen String in der CODETABLE Text mit der
#  Funktion PUTCHAR aus. Auf dem Stapel wird der Index auf das erste Zeichen
#  des Strings in Text erwartet.
#

                     # n liegt auf dem Stapel
   BEGIN
      DUP Text       # ermittle Text[n], erhalte n
   DUP WHILE         # weiter bis Text[n]=0, erhalte Text[n]
      PUTCHAR        # gebe Das Zeichen aus
      1 +            # nchstes Zeichen
   REPEAT
   DROP              # Text[n] liegt noch auf dem Stapel
   DROP              # n liegt noch auf dem Stapel

;


: PrintString ( n -> )
#
#  Gibt den String mit der Nummer n, der in der CODETABLE Text definiert ist,
#  mit der Funktion PutChar aus. n=0 bezeichnet den ersten String in der
#  Tabelle.
#

  DUP IF                   # n <> 0 ?
     0 SWAP                # 0 ist der Startindex fr SkipText
     DO SkipText -LOOP     # n Strings berspringen
  ENDIF
  PrintText

;

