#
#  VARDEF.X definiert Makros zum automatischen Plazieren von
#  Variablen im Speicher des Microcontrollers.
#
#  Folgende Makros sind in dieser Datei definiert:
#
#  IBYTE <Name>        reserviert ein Byte im internen Speicher
#                      und definiert <Name> als VARIABLE mit der
#                      reservierten Adresse.
#  IWORD <Name>        reserviert zwei Bytes fr eine 16-Bit
#                      Variable und definiert <Name> als VARIABLE
#                      mit der (unteren) reservierten Adresse.
#  <n> IDATA <Name>    reserviert <n> Bytes fr ein Array oder
#                      eine Struktur und definiert <Name> als
#                      VARIABLE mit der niedersten reservierten
#                      Adresse. <n> wird vom Rechenstapel
#                      genommen.
#  FBYTE <Name>        reserviert ein Byte im internen Speicher
#                      und definiert <Name> als FIXVAR mit der
#                      reservierten Adresse.
#  FBIT <Name>         reserviert ein Bit im bitadressierbaren
#                      Speicherbereich und definiert <Name> als
#                      BITVAR mit der reservierten Adresse.
#  XBYTE <Name>        reserviert ein Byte im externen Speicher
#                      und definiert <Name> als XVARIABLE mit der
#                      reservierten Adresse.
#  XWORD <Name>        reserviert zwei Bytes fr eine 16-Bit
#                      Variable und definiert <Name> als XVARIABLE
#                      mit der (unteren) reservierten Adresse.
#  <d> XDATA <Name>    reserviert <d> Bytes fr ein Array oder
#                      eine Struktur und definiert <Name> als
#                      XVARIABLE mit der niedersten reservierten
#                      Adresse. <d> ist ein 16-Bit Wert und wird
#                      vom Rechenstapel genommen.
# <adr> SFR <Name>     ist ein Synonym fr FIXVAR zur Deklaration
#                      von Special Function Registern bei der MCS-
#                      51 Microcontrollerfamilie.
# <adr> SBIT <Name>    ist ein Synonym fr BITVAR zur Deklaration
#                      von Bits in den Special Function Registern
#                      bei der MCS-51 Microcontrollerfamilie.
#
#  Die Makros machen es sich zunutze, da die Speicherzellen 0 und
#  1 bzw 4 und 5 des internen Speichers in MCS-51 Microcontrollern
#  fr PFX interne Daten reserviert sind. Diese Speicherzellen knnen
#  also im Interpreter verwendet werden, fr Variablen des Programms
#  werden sie jedoch sicher nicht belegt. Hier werden sie als
#  Plazierungszeiger fr Variablen eingesetzt.
#
#  Copyright (C) 1994 Markus Bauer
#

80H 0 !   # Interne Variablen unterhalb von 80H, fallende Adressen
80H 1 !   # Bitvariablen unterhalb von 80H, fallende Adressen
0L  4 !   # Externe Variablen beginnen bei 0, steigende Adressen

ALIAS\ 0 @ 1 - DUP 0 ! VARIABLE \                       IBYTE
ALIAS\ 0 @ 2 - DUP 0 ! VARIABLE \                       IWORD
ALIAS\ 0 @ SWAP - DUP 0 ! VARIABLE \                    IDATA

ALIAS\ 4 D@ OVER OVER 1L D+ 4 D! XVARIABLE \            XBYTE
ALIAS\ 4 D@ OVER OVER 2L D+ 4 D! XVARIABLE \            XWORD
ALIAS\ 4 D@ >R SWAP >R SWAP D+ 4 D! R> R> XVARIABLE \   XDATA

ALIAS\ 0 @ 1 - DUP 0 ! FIXVAR \                         FBYTE
ALIAS\ 1 @ 1 - DUP 1 ! BITVAR \                         FBIT

ALIAS\ FIXVAR \                                         SFR
ALIAS\ BITVAR \                                         SBIT

