#
#  XCONVERT.X definiert eine Reihe von Funktionen zur Wandlung von 8- und
#  16-Bit Zahlen in hexadezimale Stringdarstellung und umgekehrt.
#
#  HexToChar ( n -> ch )             wandelt eine Stelle in ein Zeichen
#  ByteToChars ( n -> ch ch )        wandelt ein Byte in einen 2-stelligen
#                                    Hex-String
#  WordToChars ( n -> ch ch ch ch )  wandelt eine 16-Bit Zahl in einen
#                                    4-stelligen Hex-String
#  CharToHex ( n -> ch )             wandelt ein Zeichen in eine Zahl
#  CharsToByte ( ch ch -> n )        wandelt zwei Zeichen in ein Byte
#  CharsToWord ( ch ch ch ch -> n )  wandelt vier Zeichen in eine 16-Bit Zahl
#
#  Copyright (C) 1994 Markus Bauer
#



CODETABLE HexToChar " 0123456789ABCDEF" ENDTABLE
#
# Wandelt eine Hexziffer (0..F) in ein ASCII Zeichen
#




: ByteToChars ( n -> ch ch )
#
#  Wandelt ein Byte in zwei Hexziffern. Das Zeichen der hherwertigen
#  Ziffer liegt nach ByteToChars zu oberst auf dem Stapel
#

   DUP 0FH AND HexToChar      # niederwertige Ziffer
   SWAP 10H /  HexToChar      # hherwertige Ziffer

;



: WordToChars ( d -> ch ch ch ch )
#
#  Wandelt eine 16-Bit Zahl in vier Hexziffern. Das Zeichen der hchst-
#  wertigen Ziffer liegt nach WordToChars zu oberst auf dem Stapel
#

   ByteToChars               # niederwertiges Byte
   ROT ByteToChars           # hherwertiges Byte

;



: CharToHex ( char -> n )
#
#  Interpretiert ein Zeichen als Hexziffer. Es findet keine Fehlerbehandlung
#  statt, ungltige Zeichen fhren zu einem nicht definierten Ergebnis
#

   DUP 'a' >= IF          # erzwinge Groschreibung
      'a' 'A' - -
   ENDIF
   '0' -                  # Wandle Ziffer
   DUP 9 > IF             # War keine Ziffer ?
      'A' '9' 1 + - -     # Korrektur fr A...F
   ENDIF
   0FH AND                # Sicherheitshalber, anstelle von Fehlerbehandlung

;


: CharsToByte ( ch ch -> n )
#
#  CharsToByte nimmt zwei Zeichen vom Stapel und interpretiert diese als
#  Hex-Zahl. Der Wert dieser Zahl wird auf den Stapel gelegt. CharsToByte
#  erwartet die hherwertige Ziffer zu oberst auf dem Stapel
#

   CharToHex 16 *         # hherwertige Ziffer umwandeln
   SWAP CharToHex         # niederwertige Ziffer umwandeln
   OR                     # zusammenfgen

;


: CharsToWord ( ch ch ch ch -> d )
#
#  CharsToByte nimmt vier Zeichen vom Stapel und interpretiert diese als
#  Hex-Zahl. Der Wert dieser Zahl wird auf den Stapel gelegt. CharsToWord
#  erwartet die hchstwertige Ziffer zu oberst auf dem Stapel
#

   CharsToByte            # hherwertiges Byte
   ROT ROT                # Ergebnis unter die restlichen Zeichen schaffen
   CharsToByte            # niederwertiges Byte

;


